//+------------------------------------------------------------------+
//|                                                       ADX EA.mq4 |
//|                                            Copyright 2013, Wahoo |
//|                                              wahoo.mql@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2013, Wahoo"
#property link      "wahoo.mql@gmail.com"
//+------------------------------------------------------------------+
//| Input Variables                                                  |
//+------------------------------------------------------------------+
extern string  Trigger="========== Trigger Settings ==========";
extern string  Indicator_Name            = "ADXcrossesNon-repainting";
extern int     ADXcrossesPeriod          = 14;

extern string  Trading="========== Trading Settings ==========";
extern double  Take_Profit               = 100.0;                          // Take Profit value in pips, if 0 - not applied
extern double  Stop_Loss                 = 100.0;                          // Stop Loss value in pips, if 0 - not applied
extern bool    Invisible_Stop_Levels     = false;                          // Enables Virtual Stops mode(Stop Levels which are not visible for broker)
extern bool    Show_Stop_Levels_on_Chart = true;                           // Shows Stop Levels on chart when Invisible Stops mode is enabled
extern double  Lot                       = 0.1;                            // Fixed lot in Lots, or % of balance in case if Auto Lot is enabled
extern bool    Auto_Lot                  = false;                          // Lot is calculated as % from balance. The bigger balance is, the bigger the lot
extern bool    Enable_Martingail         = false;                          // Enables Martingail - multiplying the lot after every loosing trade
extern double  Martingail_Multiplier     = 2.0;                            // Martingail Multiplier
extern bool    Enable_Reverse_Trades     = false;                          // Buys instead of sell and sells instead of buy
extern bool    Close_at_Opposite_Signal  = true;                           // Closes buys when sell signal and sells when buy signal
extern int     Max_Orders                = 1;                              // Maximum allowed number of trades
extern int     Signal_Bar                = 1;                              // Signal Bar Number

extern string  Trailing="========== Trailing Stop Settings ==========";
extern bool    Enable_Trailing_Stop      = false;                          // Enables Trailing Stop
extern double  Trailing_Stop_Start_Level = 50.0;                           // Trailing Stop Activation Level in pips
extern double  Trailing_Stop_Distance    = 20.0;                           // Trailing Stop Distance in pips

extern string  BreakEven="========== Break Even Settings ==========";
extern bool    Enable_Break_Even         = false;                          // Enables Break-Even
extern double  Break_Even_Level          = 30.0;                           // Break-Even Activation Level in pips
extern double  Break_Even_Profit         = 1.0;                            // Break-Even Profit level in pips

extern string  TimeFilters="========== Time Settings ==========";
extern bool    Use_Time_Zone_1           = false;                          // Enables First Time Zone
extern string  Time_Zone_1_Start         = "08:00";                        // First Time Zone start time 
extern string  Time_Zone_1_End           = "12:00";                        // First Time Zone end time 
extern bool    Use_Time_Zone_2           = false;                          // Enables second Time Zone
extern string  Time_Zone_2_Start         = "14:00";                        // Second Time Zone start time 
extern string  Time_Zone_2_End           = "16:00";                        // Second Time Zone end time 
extern bool    Close_Trades_on_Friday    = false;                          // Enables closing on Friday 
extern string  Friday_Close_Time         = "22:30";                        // Close on Friday time

extern string  General="========== EA General Settings ==========";
extern double  Slippage                  = 2.0;                            // Maximum allowed slippage
extern string  EA_Orders_Comment         = "ADX EA";                       // Orders Comments
extern int     Magic                     = 57575;                          // EA magic number
extern bool    ECN_Broker                = true;                           // ECN brokers support
extern bool    Journaling                = true;                           // Enables extended messages about EA activities
extern bool    Alert_Journal             = false;                          // Alerts EA messages
extern bool    eMail_Journal             = false;                          // E-mails EA messages
extern bool    Push_Journal              = false;                          // Enables push notifications
extern bool    Draw_Closed_Orders        = false;                          // Enables Drawing of closed orders on chart
extern bool    Show_Comments             = true;                           // Shows some comments on chart

extern string  Errors="========== Errors Handling Settings ==========";
extern int     Retry_Interval            = 100;                            // Retry time in milliseconds in case of error or busy trading context
extern int     Max_Retries_Per_Tick      = 2;                              // Number of times EA will try to open an order during one tick
//+------------------------------------------------------------------+
//| End of Input Variables                                           |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Service Variables                                                |
//+------------------------------------------------------------------+
double Trailing_Stop_Step=0.1;
double P=1.0;
int    OrdersHistory=0;
double Stop=0;
double Take=0;
string prefix="";
//+------------------------------------------------------------------+
//| End of Service Variables                                         |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//===== Handle Digits
   if(Digits==5 || Digits==3)P=10;else P=1;

//===== Prefix for all EA objects and Global Variables
   if(IsTesting())
      prefix="Test"+"_"+Magic+"_"+Symbol()+"_"+AccountNumber()+"_";
   else
      prefix=Magic+"_"+Symbol()+"_"+AccountNumber()+"_";

//===== Handle Stops if Invisible Stops activated
   if(!Invisible_Stop_Levels)
     {
      Stop=Stop_Loss;
      Take=Take_Profit;
     }
   else
     {
      Stop=0;
      Take=0;
     }

//===== Delete Objects and Global Variables
   if(IsTesting())
      DeleteGlobalVariables();
   DeleteObjects();

//=====   
   return(0);
  }
//+------------------------------------------------------------------+
//| End of expert initialization function                            |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
//===== Manage orders  
   TrailingStopAll();
   BreakEvenAll();
   VirtualTrailingStopAll();
   VirtualBreakEvenAll();
   VirtualStopsAll();
   DrawClosedOrders();

//===== Allow Trade only once a bar
   if(NewBar())GVDel("Trade");

//===== Close all on Friday Functionality
   if(CloseAllFriday())
     CloseAllPositions("Close all on Friday");
     

//===== Signals
   bool buy=false;
   bool sell=false;
  

   if(
      ADXCross(0,Signal_Bar)>0 && ADXCross(0,Signal_Bar)!=EMPTY_VALUE
      )
     {
      if(!Enable_Reverse_Trades)buy=true;
      if(Enable_Reverse_Trades)sell=true;
     }

   if(
      ADXCross(1,Signal_Bar)>0 && ADXCross(1,Signal_Bar)!=EMPTY_VALUE
      )
     {
      if(!Enable_Reverse_Trades)sell=true;
      if(Enable_Reverse_Trades)buy=true;
     }
   

//===== Trading
   if(buy && !sell)
     {
      if(Close_at_Opposite_Signal)
         KillOrdersType(OP_SELL,"Close at opposite signal");
      if(GoodTime() && CountAllPositions()<Max_Orders && GVGet("Trade")==0)
        {
         if(MarketOrder(OP_BUY,GetLot(),Stop,Take,"BUY signal")>0)
            GVSet("Trade",1);
        }
     }
   if(sell && !buy)
     {
      if(Close_at_Opposite_Signal)
         KillOrdersType(OP_BUY,"Close at opposite signal");
      if(GoodTime() && CountAllPositions()<Max_Orders && GVGet("Trade")==0)
        {
         if(MarketOrder(OP_SELL,GetLot(),Stop,Take,"Sell signal")>0)
            GVSet("Trade",1);
        }
     }

//=====
   ShowComments();

    return(0);
  }
//+------------------------------------------------------------------+
//| End of expert start function                                     |
//+------------------------------------------------------------------+
double ADXCross(int buf,int bar)
{
return(iCustom(NULL,0,Indicator_Name,ADXcrossesPeriod,buf,bar));
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//=====
   if(IsTesting())
      DeleteGlobalVariables();
   else
      DeleteObjects();

   if(Show_Comments)
      Comment("");

//=====
   return(0);
  }
//+------------------------------------------------------------------+
//| End of expert deinitialization function                          |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Show Comments                                                    |
//+------------------------------------------------------------------+
void ShowComments()
  {
   if(Show_Comments)
     {
      string comm="";
      comm=comm+EA_Orders_Comment+"\n"+"\n";
      comm=comm+"Last Server Time: "+TimeToStr(TimeCurrent(),TIME_DATE | TIME_MINUTES | TIME_SECONDS)+"\n";
      comm=comm+"Last Spread: "+DoubleToStr((Ask-Bid)/(Point*P),1)+" pips"+"\n";
      comm=comm+"Account Balance: "+DoubleToStr(AccountBalance(),2)+"\n";
      comm=comm+"Equity: "+DoubleToStr(AccountEquity(),2)+"\n";
      comm=comm+"Free Margin: "+DoubleToStr(AccountFreeMargin(),2)+"\n";
    
      if(CountAllPositions()==0)
         comm=comm+"Currently there is NO open orders"+"\n";
      else
        {
         if(CountOrdersType(OP_BUY)>0)
            comm=comm+"Open BUYs: "+CountOrdersType(OP_BUY)+"\n";
         if(CountOrdersType(OP_SELL)>0)
            comm=comm+"Open SELLs: "+CountOrdersType(OP_SELL)+"\n";
        }
      Comment(comm);
     }
  }
//+------------------------------------------------------------------+
//| End of Show Comments                                             |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Time limited trading                                             |
//+------------------------------------------------------------------+
bool GoodTime()
  {
   if(CloseAllFriday())return(false);
   if(!Use_Time_Zone_1 && !Use_Time_Zone_2)return(true);
   if(Use_Time_Zone_1 && TimeCurrent()>StrToTime(Time_Zone_1_Start) && TimeCurrent()<StrToTime(Time_Zone_1_End))return(true);
   if(Use_Time_Zone_2 && TimeCurrent()>StrToTime(Time_Zone_2_Start) && TimeCurrent()<StrToTime(Time_Zone_2_End))return(true);
   return(false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CloseAllFriday()
  {
   if(Close_Trades_on_Friday && TimeDayOfWeek(TimeCurrent())==5 && TimeCurrent()>=StrToTime(Friday_Close_Time))
      return(true); else return(false);
  }
//+------------------------------------------------------------------+
//| Time limited trading                                             |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| New Bar                                                          |
//+------------------------------------------------------------------+
bool NewBar(int Time_Frame=0)
  {
   if(iTime(NULL,Time_Frame,0)!=GVGet("LastBar"))
     {
      GVSet("LastBar",iTime(NULL,Time_Frame,0));
      return(true);
     }
   else return(false);
  }
//+------------------------------------------------------------------+
//| End of New Bar                                                   |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Informer                                                         |
//+------------------------------------------------------------------+
void Informer(string MESSAGE)
  {
   if(Journaling)Print(MESSAGE);
   if(eMail_Journal)SendMail(EA_Orders_Comment,MESSAGE);
   if(Push_Journal)SendNotification(MESSAGE);
   if(Alert_Journal)Alert(MESSAGE);
  }
//+------------------------------------------------------------------+
//| End of Informer                                                  |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| OPEN FROM MARKET // CURRENT SYMBOL // EA MAGIC // EA Comment     |
//+------------------------------------------------------------------+
int MarketOrder(int TYPE,double LOT,double SL=0,double TP=0,string reason="",string symb="")
  {
   int tries=0;
   string symbol=symb;
   if(symbol=="")symbol=Symbol();

   int cmd=TYPE;

   double volume=CheckLot(LOT,symbol);
   if(MarketInfo(symbol,MODE_MARGINREQUIRED)*volume>AccountFreeMargin())
     {
      Informer("Can not open a trade. Not enough free margin to open "+volume+" on "+symbol);
      return(-1);
     }
   int slippage=Slippage*P;
   string comment=EA_Orders_Comment;
   int magic=Magic;
   datetime expiration=0;
   color arrow_color=0;if(TYPE==OP_BUY)arrow_color=Blue;if(TYPE==OP_SELL)arrow_color=Green;
   double stoploss=0;
   double takeprofit=0;

   if(SL!=0 && SL<=MathCeil(MarketInfo(symbol,MODE_STOPLEVEL)/P))SL=MathCeil(MarketInfo(symbol,MODE_STOPLEVEL)/P);
   if(TP!=0 && TP<=MathCeil(MarketInfo(symbol,MODE_STOPLEVEL)/P))TP=MathCeil(MarketInfo(symbol,MODE_STOPLEVEL)/P);

   int Ticket=-1;
   double price=0;
   double point=MarketInfo(symbol,MODE_POINT);
   int digits=MarketInfo(symbol,MODE_DIGITS);
   double ask=MarketInfo(symbol,MODE_ASK);
   double bid=MarketInfo(symbol,MODE_BID);

   if(!ECN_Broker || (TP==0 && SL==0))
     {
      while(tries<Max_Retries_Per_Tick)
        {
         RefreshRates();
         ask=MarketInfo(symbol,MODE_ASK);
         bid=MarketInfo(symbol,MODE_BID);

         if(TYPE==OP_BUY)price=ask;if(TYPE==OP_SELL)price=bid;

         if(TYPE==OP_BUY && SL!=0)
           {
            stoploss=NormalizeDouble(ask-SL*P*point,digits);
            if(bid>stoploss && bid-stoploss<=MarketInfo(symbol,MODE_STOPLEVEL)*point)stoploss=NormalizeDouble(bid-MarketInfo(symbol,MODE_STOPLEVEL)*point,digits);
           }
         if(TYPE==OP_SELL && SL!=0)
           {
            stoploss=NormalizeDouble(bid+SL*P*point,digits);
            if(stoploss>ask && stoploss-ask<=MarketInfo(symbol,MODE_STOPLEVEL)*point)stoploss=NormalizeDouble(ask+MarketInfo(symbol,MODE_STOPLEVEL)*point,digits);
           }
         if(TYPE==OP_BUY && TP!=0)
           {
            takeprofit=NormalizeDouble(ask+TP*P*point,digits);
            if(takeprofit>ask && takeprofit-ask<=MarketInfo(symbol,MODE_STOPLEVEL)*point)takeprofit=NormalizeDouble(ask+MarketInfo(symbol,MODE_STOPLEVEL)*point,digits);
           }
         if(TYPE==OP_SELL && TP!=0)
           {
            takeprofit=NormalizeDouble(bid-TP*P*point,digits);
            if(bid>takeprofit && bid-takeprofit<=MarketInfo(symbol,MODE_STOPLEVEL)*point)takeprofit=NormalizeDouble(bid-MarketInfo(symbol,MODE_STOPLEVEL)*point,digits);
           }
         Informer("EA Journaling: Trying to place a market order... "+reason);
         if(HandleTradingEnvironment())
           {
            ask=MarketInfo(symbol,MODE_ASK);
            bid=MarketInfo(symbol,MODE_BID);
            if(TYPE==OP_BUY)price=ask;
            if(TYPE==OP_SELL)price=bid;
            Ticket=OrderSend(symbol,cmd,volume,NormalizeDouble(price,digits),slippage,stoploss,takeprofit,comment,magic,expiration,arrow_color);
           }
         if(Ticket>0)break;
         tries++;
         Sleep(Retry_Interval);
        }
     }
   if(ECN_Broker && (TP!=0 || SL!=0))
     {
      while(tries<Max_Retries_Per_Tick)
        {
         Informer("EA Journaling: Trying to place a market order... "+reason);
         if(HandleTradingEnvironment())
           {
            ask=MarketInfo(symbol,MODE_ASK);
            bid=MarketInfo(symbol,MODE_BID);
            if(TYPE==OP_BUY)price=ask;
            if(TYPE==OP_SELL)price=bid;
            Ticket=OrderSend(symbol,cmd,volume,NormalizeDouble(price,digits),slippage,0,0,comment,magic,expiration,arrow_color);
           }
         if(Ticket>0)break;
         tries++;
         Sleep(Retry_Interval);
        }
      if(Ticket>0 && OrderSelect(Ticket,SELECT_BY_TICKET)==true && (SL!=0 || TP!=0))
        {
         double openprice=OrderOpenPrice();
         if(TYPE==OP_BUY && SL!=0)
           {
            stoploss=NormalizeDouble(openprice-SL*P*point,digits);
            if(bid>stoploss && bid-stoploss<=MarketInfo(symbol,MODE_STOPLEVEL)*point)stoploss=NormalizeDouble(bid-MarketInfo(symbol,MODE_STOPLEVEL)*point,digits);
           }
         if(TYPE==OP_SELL && SL!=0)
           {
            stoploss=NormalizeDouble(openprice+SL*P*point,digits);
            if(stoploss>ask && stoploss-ask<=MarketInfo(symbol,MODE_STOPLEVEL)*point)stoploss=NormalizeDouble(ask+MarketInfo(symbol,MODE_STOPLEVEL)*point,digits);
           }
         if(TYPE==OP_BUY && TP!=0)
           {
            takeprofit=NormalizeDouble(openprice+TP*P*point,digits);
            if(takeprofit>ask && takeprofit-ask<=MarketInfo(symbol,MODE_STOPLEVEL)*point)takeprofit=NormalizeDouble(ask+MarketInfo(symbol,MODE_STOPLEVEL)*point,digits);
           }
         if(TYPE==OP_SELL && TP!=0)
           {
            takeprofit=NormalizeDouble(openprice-TP*P*point,digits);
            if(bid>takeprofit && bid-takeprofit<=MarketInfo(symbol,MODE_STOPLEVEL)*point)takeprofit=NormalizeDouble(bid-MarketInfo(symbol,MODE_STOPLEVEL)*point,digits);
           }

         bool ModifyOpen=false;
         tries=0;
         while(tries<Max_Retries_Per_Tick)
           {
            if(HandleTradingEnvironment())
               ModifyOpen=OrderModify(Ticket,OrderOpenPrice(),stoploss,takeprofit,expiration,arrow_color);
            if(ModifyOpen)break;
            tries++;
            Sleep(Retry_Interval);
           }
        }
     }
   if(Ticket<0)Informer("EA Journaling: Unexpected Error has happened. Error Description: "+GetErrorDescription(GetLastError()));
   if(Ticket>0)Informer("EA Journaling: Order successfully placed. Ticket: "+Ticket+". "+reason);
   return(Ticket);
  }
//+------------------------------------------------------------------+
//| End of OPEN FROM MARKET//CURRENT SYMBOL//EA MAGIC//EAComment     |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| CHECK LOT // CURRENT SYMBOL                                      |
//+------------------------------------------------------------------+
double CheckLot(double Lot,string symb="")
  {
   double LotToOpen=0;
   string symbol=symb;
   if(symbol=="")symbol=Symbol();

   LotToOpen=NormalizeDouble(Lot,2);
   LotToOpen=MathCeil(LotToOpen/MarketInfo(symbol,MODE_LOTSTEP))*MarketInfo(symbol,MODE_LOTSTEP);
   if(LotToOpen<MarketInfo(symbol,MODE_MINLOT))LotToOpen=MarketInfo(symbol,MODE_MINLOT);
   if(LotToOpen>MarketInfo(symbol,MODE_MAXLOT))LotToOpen=MarketInfo(symbol,MODE_MAXLOT);
   LotToOpen=NormalizeDouble(LotToOpen,2);
   if(MathAbs(LotToOpen-Lot)>0.001)Informer("EA Journaling: Trading Lot has been changed by CheckLot function. Requested lot: "+Lot+". Lot to open: "+LotToOpen);
   return(LotToOpen);
  }
//+------------------------------------------------------------------+
//| End of CHECK LOT // CURRENT SYMBOL                               |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| AUTO LOT + Martingail                                            |
//+------------------------------------------------------------------+
double GetLot()
  {
   double lot=0;
   if(!Auto_Lot)lot=Lot;
   if(Auto_Lot)lot=AccountBalance()*(Lot/100)*AccountLeverage()/MarketInfo(Symbol(),MODE_LOTSIZE);
   if(!Enable_Martingail)return(lot);
   if(Enable_Martingail)
     {
      double lastprofit=LastClosedProfit();
      if(lastprofit>=0)return(lot);
      if(lastprofit<0)return(GetLastPositionLots(true)*Martingail_Multiplier);
     }
  }
//+------------------------------------------------------------------+
//| End of Auto Lot                                                  |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| LAST CLOSED PROFIT // CURRENT SYMBOL // EA MAGIC                 |
//+------------------------------------------------------------------+
double LastClosedProfit()
  {
   int Ticket=-1;
   datetime closetime=0;
   int ordershistory=OrdersHistoryTotal()-1;
   for(int i=ordershistory;i>=0;i--)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==true && OrderSymbol()==Symbol() && OrderMagicNumber()==Magic && OrderType()<=OP_SELL)
        {
         if(OrderCloseTime()>closetime)
           {
            Ticket=OrderTicket();
            closetime=OrderCloseTime();
           }
        }
     }
   if(OrderSelect(Ticket,SELECT_BY_TICKET)==true)return(OrderProfit());
   else return(0);
  }
//+------------------------------------------------------------------+
//|end of LAST CLOSED PROFIT//CURRENT SYMBOL//EA MAGIC               |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| CLOSE/DELETE ORDERS TYPE // CURRENT SYMBOL // EA MAGIC           |
//+------------------------------------------------------------------+
bool KillOrdersType(int TYPE,string reason="",string symb="")
  {
   string symbol=symb;
   if(symbol=="")symbol=Symbol();
   int orders=OrdersTotal()-1;
   for(int i=orders; i>=0; i--)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true && OrderSymbol()==symbol && OrderMagicNumber()==Magic && OrderType()==TYPE)
         KillTicket(OrderTicket(),reason);
     }
   if(CountOrdersType(TYPE)==0)return(true); else return(false);
  }
//+------------------------------------------------------------------+
//| CLOSE/DELETE ORDERS TYPE // CURRENT SYMBOL // EA MAGIC           |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| CLOSE ALL POSITIONS // CURRENT SYMBOL // EA MAGIC                |
//+------------------------------------------------------------------+
bool CloseAllPositions(string reason="",string symb="")
  {
   string symbol=symb;
   if(symbol=="")symbol=Symbol();
   int orders=OrdersTotal()-1;
   for(int i=orders; i>=0; i--)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true && OrderSymbol()==symbol && OrderMagicNumber()==Magic && OrderType()<=OP_SELL)
        {
         KillTicket(OrderTicket(),reason);
        }
     }
   if(CountAllPositions(symb)==0)
      return(true);
   else return(false);
  }
//+------------------------------------------------------------------+
//| End of CLOSE ALL POSITIONS // CURRENT SYMBOL // EA MAGIC         |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| KILL ORDER TICKET                                                |
//+------------------------------------------------------------------+
bool KillTicket(int Ticket,string reason="")
  {
   if(OrderSelect(Ticket,SELECT_BY_TICKET)==true && OrderType()<=OP_SELL && OrderCloseTime()==0)
     {
      bool Closing=false;
      double Price=0;
      color arrow_color=0;if(OrderType()==OP_BUY)arrow_color=Blue;if(OrderType()==OP_SELL)arrow_color=Green;
      Informer("EA Journaling: Trying to close position "+Ticket+" ... "+reason);
      if(HandleTradingEnvironment())
        {
         if(OrderType()==OP_BUY)
            Price=MarketInfo(OrderSymbol(),MODE_BID);
         if(OrderType()==OP_SELL)
            Price=MarketInfo(OrderSymbol(),MODE_ASK);
         Closing=OrderClose(Ticket,OrderLots(),Price,Slippage*P,arrow_color);
        }
      if(!Closing)Informer("EA Journaling: Unexpected Error has happened. Error Description: "+GetErrorDescription(GetLastError()));
      if(Closing)Informer("EA Journaling: Position successfully closed. "+reason);
      return(Closing);
     }
   if(OrderSelect(Ticket,SELECT_BY_TICKET)==true && OrderType()>OP_SELL && OrderCloseTime()==0)
     {
      bool Delete=false;
      Informer("EA Journaling: Trying to delete pending order "+Ticket+" ... "+reason);
      if(HandleTradingEnvironment())
         Delete=OrderDelete(Ticket,CLR_NONE);
      if(!Delete)Informer("EA Journaling: Unexpected Error has happened. Error Description: "+GetErrorDescription(GetLastError()));
      if(Delete)Informer("EA Journaling: Order successfully deleted. "+reason);
      return(Delete);
     }
   return(true);
  }
//+------------------------------------------------------------------+
//| End of KILL ORDER TICKET                                         |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| COUNT ALL POSITIONS // CURRENT SYMBOL // EA MAGIC                |
//+------------------------------------------------------------------+
int CountAllPositions(string symb="")
  {
   string symbol=symb;
   if(symbol=="")symbol=Symbol();

   int Positions=0;
   int orders=OrdersTotal()-1;
   for(int i=orders; i>=0; i--)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true && OrderSymbol()==symbol && OrderMagicNumber()==Magic && OrderType()<=OP_SELL)
         Positions++;
     }
   return(Positions);
  }
//+------------------------------------------------------------------+
//| End of COUNT ALL POSITIONS // CURRENT SYMBOL // EA MAGIC         |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| COUNT POSITIONS TYPE // CURRENT SYMBOL // EA MAGIC               |
//+------------------------------------------------------------------+
int CountOrdersType(int TYPE)
  {
   int Orders=0;
   int orders=OrdersTotal()-1;
   for(int i=orders; i>=0; i--)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true && OrderSymbol()==Symbol() && OrderMagicNumber()==Magic && OrderType()==TYPE)
         Orders++;
     }
   return(Orders);
  }
//+------------------------------------------------------------------+
//| End of COUNT POSITIONS TYPE // CURRENT SYMBOL // EA MAGIC        |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| LAST OPENED POSITION LOTS // CURRENT SYMBOL // EA MAGIC          |
//+------------------------------------------------------------------+
double GetLastPositionLots(bool closedonly)
  {
   int Ticket=-1;
   double lot=0;
   datetime opentime=0;
   if(!closedonly)
     {
      int orders=OrdersTotal()-1;
      for(int i=orders; i>=0; i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true && OrderSymbol()==Symbol() && OrderMagicNumber()==Magic && OrderType()<=OP_SELL)
           {
            if(OrderOpenTime()>opentime)
              {
               Ticket=OrderTicket();
               opentime=OrderOpenTime();
              }
           }
        }
     }
   int ordershistory=OrdersHistoryTotal()-1;
   for(int j=ordershistory;j>=0;j--)
     {
      if(OrderSelect(j,SELECT_BY_POS,MODE_HISTORY)==true && OrderSymbol()==Symbol() && OrderMagicNumber()==Magic && OrderType()<=OP_SELL)
        {
         if(OrderOpenTime()>opentime)
           {
            Ticket=OrderTicket();
            opentime=OrderOpenTime();
           }
        }
     }
   if(OrderSelect(Ticket,SELECT_BY_TICKET))lot=OrderLots();
   return(lot);
  }
//+------------------------------------------------------------------+
//| End of LAST OPENED POSITION LOTS // CURRENT SYMBOL // EA MAGIC   |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Draw Closed Orders                                               |
//+------------------------------------------------------------------+
void DrawClosedOrders()
  {
   int history=OrdersHistoryTotal();
   if(Draw_Closed_Orders && history!=OrdersHistory)
     {
      for(int i=history-1; i>=0; i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY) && OrderSymbol()==Symbol() && OrderMagicNumber()==Magic && OrderType()<=OP_SELL)
           {
            if(OrderType()==OP_BUY)
               CreateTLine(OrderTicket(),Blue,OrderOpenTime(),OrderOpenPrice(),OrderCloseTime(),OrderClosePrice(),false);

            if(OrderType()==OP_SELL)
               CreateTLine(OrderTicket(),Green,OrderOpenTime(),OrderOpenPrice(),OrderCloseTime(),OrderClosePrice(),false);
           }
        }
      OrdersHistory=history;
     }
  }
//+------------------------------------------------------------------+
//| End of Draw Closed Orders                                        |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Delete Objects                                                   |
//+------------------------------------------------------------------+
void DeleteObjects()
  {
   int obj=ObjectsTotal()-1;
   for(int o=obj;o>=0;o--)
     {
      string name=ObjectName(o);
      int index=StringFind(name,prefix,0);
      if(index>-1)
         ObjectDelete(name);
     }
  }
//+------------------------------------------------------------------+
//| End of Delete Objects                                            |
//+------------------------------------------------------------------+  
//+------------------------------------------------------------------+
//| Delete Global Variables                                          |
//+------------------------------------------------------------------+
void DeleteGlobalVariables()
  {
   int obj=GlobalVariablesTotal()-1;
   for(int o=obj; o>=0;o--)
     {
      string name=GlobalVariableName(o);
      int index=StringFind(name,prefix,0);
      if(index>-1)
         GlobalVariableDel(name);
     }
  }
//+------------------------------------------------------------------+
//| End of Delete Global Variables                                   |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+  
datetime GVSet(string name,double value)
  {
   return(GlobalVariableSet(prefix+name,value));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GVGet(string name)
  {
   return(GlobalVariableGet(prefix+name));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool GVDel(string name)
  {
   return(GlobalVariableDel(prefix+name));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CreateTLine(string NAME,color col,datetime time1,double price1,datetime time2,double price2,bool ray=true,int window=0,int width=2)
  {
   string name=prefix+NAME;
   if(ObjectFind(name)!=window)
      ObjectCreate(name,OBJ_TREND,window,time1,price1,time2,price2);

   ObjectSet(name,OBJPROP_WIDTH,width);
   ObjectSet(name,OBJPROP_COLOR,col);
   ObjectSet(name,OBJPROP_STYLE,STYLE_SOLID);
   ObjectSet(name,OBJPROP_TIME1,time1);
   ObjectSet(name,OBJPROP_TIME2,time2);
   ObjectSet(name,OBJPROP_PRICE1,price1);
   ObjectSet(name,OBJPROP_PRICE2,price2);
   ObjectSet(name,OBJPROP_RAY,ray);

   if(ObjectFind(name)==window)return(true);
   return(false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CreateText(string NAME,int window,datetime time,double price,string text,int size,string font,color col)
  {
   string name=prefix+NAME;
   if(ObjectFind(name)!=window)
      ObjectCreate(name,OBJ_TEXT,window,time,price);

   ObjectSet(name,OBJPROP_TIME1,time);
   ObjectSet(name,OBJPROP_PRICE1,price);
   ObjectSetText(name,text,size,font,col);

   if(ObjectFind(name)==window)return(true);
   return(false);
  }
//+------------------------------------------------------------------+
//| VIRTUAL STOPS for ALL POSITIONS//CURRENT SYMBOL//EA MAGIC        |
//+------------------------------------------------------------------+
void VirtualStopsAll(string symb="")
  {
   if(Invisible_Stop_Levels)
     {
      string symbol=symb;
      if(symbol=="")symbol=Symbol();
      int orders=OrdersTotal()-1;
      for(int i=orders; i>=0; i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true && OrderSymbol()==symbol && OrderMagicNumber()==Magic)
           {
            int ticket=OrderTicket();
            VirtualStopsOne(ticket);
           }
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void VirtualStopsOne(int TICKET)
  {
   if(Invisible_Stop_Levels)
     {
      if(OrderSelect(TICKET,SELECT_BY_TICKET)==true && OrderCloseTime()==0)
        {
         bool Closing=false;
         string symbol=OrderSymbol();
         double point=MarketInfo(symbol,MODE_POINT);
         int digits=MarketInfo(symbol,MODE_DIGITS);
         double ask=MarketInfo(symbol,MODE_ASK);
         double bid=MarketInfo(symbol,MODE_BID);

         if(OrderType()==OP_BUY && ((GVGet(TICKET+"VirtStop")!=0 && bid<=GVGet(TICKET+"VirtStop")) || (GVGet(TICKET+"VirtTake")!=0 && bid>=GVGet(TICKET+"VirtTake"))))
            Closing=KillTicket(TICKET,"Virtual Stop level reached");
         if(Closing)
           {
            GVDel(TICKET+"VirtStop");
            GVDel(TICKET+"VirtTake");
           }
         if(OrderType()==OP_SELL && ((GVGet(TICKET+"VirtStop")!=0 && ask>=GVGet(TICKET+"VirtStop")) || (GVGet(TICKET+"VirtTake")!=0 && ask<=GVGet(TICKET+"VirtTake"))))
            Closing=KillTicket(TICKET,"Virtual Stop level reached");
         if(Closing)
           {
            GVDel(TICKET+"VirtStop");
            GVDel(TICKET+"VirtTake");
           }
         RefreshRates();
         if(OrderType()==OP_BUY && Stop_Loss!=0 && GVGet(TICKET+"VirtStop")==0)
           {
            GVSet(TICKET+"VirtStop",NormalizeDouble(OrderOpenPrice()-Stop_Loss*P*point,digits));

            if(Show_Stop_Levels_on_Chart)
               CreateText(TICKET+"VirtStop",0,TimeCurrent(),GVGet(TICKET+"VirtStop"),"VirtStop Order "+TICKET+": "+DoubleToStr(GVGet(TICKET+"VirtStop"),digits),5,"Tahoma",Blue);
           }
         if(OrderType()==OP_SELL && Stop_Loss!=0 && GVGet(TICKET+"VirtStop")==0)
           {
            GVSet(TICKET+"VirtStop",NormalizeDouble(OrderOpenPrice()+Stop_Loss*P*point,digits));

            if(Show_Stop_Levels_on_Chart)
               CreateText(TICKET+"VirtStop",0,TimeCurrent(),GVGet(TICKET+"VirtStop"),"VirtStop Order "+TICKET+": "+DoubleToStr(GVGet(TICKET+"VirtStop"),digits),5,"Tahoma",Blue);
           }
         if(OrderType()==OP_BUY && Take_Profit!=0 && GVGet(TICKET+"VirtTake")==0)
           {
            GVSet(TICKET+"VirtTake",NormalizeDouble(OrderOpenPrice()+Take_Profit*P*point,digits));

            if(Show_Stop_Levels_on_Chart)
               CreateText(TICKET+"VirtTake",0,TimeCurrent(),GVGet(TICKET+"VirtTake"),"VirtTake Order "+TICKET+": "+DoubleToStr(GVGet(TICKET+"VirtTake"),digits),5,"Tahoma",Blue);
           }
         if(OrderType()==OP_SELL && Take_Profit!=0 && GVGet(TICKET+"VirtTake")==0)
           {
            GVSet(TICKET+"VirtTake",NormalizeDouble(OrderOpenPrice()-Take_Profit*P*point,digits));

            if(Show_Stop_Levels_on_Chart)
               CreateText(TICKET+"VirtTake",0,TimeCurrent(),GVGet(TICKET+"VirtTake"),"VirtTake Order "+TICKET+": "+DoubleToStr(GVGet(TICKET+"VirtTake"),digits),5,"Tahoma",Blue);
           }
        }
     }
  }
//+------------------------------------------------------------------+
//| End of VIRTUAL STOPS for ALL POSITIONS//CURRENT SYMBOL//EA MAGIC |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| VIRTUAL BREAK EVEN for ALL POSITIONS//CURRENT SYMBOL//EA MAGIC   |
//+------------------------------------------------------------------+
void VirtualBreakEvenAll(string symb="")
  {
   if(Enable_Break_Even && Invisible_Stop_Levels)
     {
      string symbol=symb;
      if(symbol=="")symbol=Symbol();
      int orders=OrdersTotal()-1;
      for(int i=orders; i>=0; i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true && OrderSymbol()==symbol && OrderMagicNumber()==Magic)
           {
            int ticket=OrderTicket();
            VirtualBreakEvenOne(ticket);
           }
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void VirtualBreakEvenOne(int TICKET)
  {
   if(!Enable_Break_Even || !Invisible_Stop_Levels)return;
   if(OrderSelect(TICKET,SELECT_BY_TICKET) && OrderCloseTime()==0)
     {
      bool Closing=false;
      string symbol=OrderSymbol();
      double point=MarketInfo(symbol,MODE_POINT);
      int digits=MarketInfo(symbol,MODE_DIGITS);
      double ask=MarketInfo(symbol,MODE_ASK);
      double bid=MarketInfo(symbol,MODE_BID);

      if(OrderType()==OP_BUY && GVGet(TICKET+"VirtStop")!=0 && bid<=GVGet(TICKET+"VirtStop"))
         Closing=KillTicket(TICKET,"Virtual Stop level reached");
      if(Closing)
        {
         GVDel(TICKET+"VirtStop");
         GVDel(TICKET+"VirtTake");
        }
      if(OrderType()==OP_SELL && GVGet(TICKET+"VirtStop")!=0 && ask>=GVGet(TICKET+"VirtStop"))
         Closing=KillTicket(TICKET,"Virtual Stop level reached");
      if(Closing)
        {
         GVDel(TICKET+"VirtStop");
         GVDel(TICKET+"VirtTake");
        }
      RefreshRates();
      if(OrderType()==OP_BUY && (bid-OrderOpenPrice())>=(Break_Even_Level*P*point) && (GVGet(TICKET+"VirtStop")<NormalizeDouble(OrderOpenPrice()+Break_Even_Profit*P*point,digits) || GVGet(TICKET+"VirtStop")==0))
        {
         GVSet(TICKET+"VirtStop",NormalizeDouble(OrderOpenPrice()+Break_Even_Profit*P*point,digits));

         if(Show_Stop_Levels_on_Chart)
            CreateText(TICKET+"VirtStop",0,TimeCurrent(),GVGet(TICKET+"VirtStop"),"VirtStop Order "+TICKET+": "+DoubleToStr(GVGet(TICKET+"VirtStop"),digits),5,"Tahoma",Blue);
        }
      if(OrderType()==OP_SELL && (OrderOpenPrice()-ask)>=(Break_Even_Level*P*point) && (GVGet(TICKET+"VirtStop")>NormalizeDouble(OrderOpenPrice()-Break_Even_Profit*P*point,digits) || GVGet(TICKET+"VirtStop")==0))
        {
         GVSet(TICKET+"VirtStop",NormalizeDouble(OrderOpenPrice()-Break_Even_Profit*P*point,digits));

         if(Show_Stop_Levels_on_Chart)
            CreateText(TICKET+"VirtStop",0,TimeCurrent(),GVGet(TICKET+"VirtStop"),"VirtStop Order "+TICKET+": "+DoubleToStr(GVGet(TICKET+"VirtStop"),digits),5,"Tahoma",Blue);
        }
     }
  }
//+------------------------------------------------------------------+
//| End of BREAK EVEN for ALL POSITIONS // CURRENT SYMBOL // EA MAGIC|
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| VIRTUAL TRAILING STOP for ALL POSITIONS//CURRENT SYMBOL//EA MAGIC|
//+------------------------------------------------------------------+
void VirtualTrailingStopAll(string symb="")
  {
   if(Enable_Trailing_Stop && Invisible_Stop_Levels)
     {
      string symbol=symb;
      if(symbol=="")symbol=Symbol();
      int orders=OrdersTotal()-1;
      for(int i=orders; i>=0; i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true && OrderSymbol()==symbol && OrderMagicNumber()==Magic)
           {
            int ticket=OrderTicket();
            VirtualTrailingStopOne(ticket);
           }
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void VirtualTrailingStopOne(int TICKET)
  {
   if(!Enable_Trailing_Stop || !Invisible_Stop_Levels)return;
   if(OrderSelect(TICKET,SELECT_BY_TICKET) && OrderCloseTime()==0)
     {
      string symbol=OrderSymbol();
      double point=MarketInfo(symbol,MODE_POINT);
      int digits=MarketInfo(symbol,MODE_DIGITS);
      double ask=MarketInfo(symbol,MODE_ASK);
      double bid=MarketInfo(symbol,MODE_BID);

      bool Closing=false;
      if(OrderType()==OP_BUY && GVGet(TICKET+"VirtStop")!=0 && bid<=GVGet(TICKET+"VirtStop"))
         Closing=KillTicket(TICKET,"Virtual Stop level reached");
      if(Closing)
        {
         GVDel(TICKET+"VirtStop");
         GVDel(TICKET+"VirtTake");
        }
      if(OrderType()==OP_SELL && GVGet(TICKET+"VirtStop")!=0 && ask>=GVGet(TICKET+"VirtStop"))
         Closing=KillTicket(TICKET,"Virtual Stop level reached");
      if(Closing)
        {
         GVDel(TICKET+"VirtStop");
         GVDel(TICKET+"VirtTake");
        }
      RefreshRates();
      if(OrderType()==OP_BUY && (bid-OrderOpenPrice())>=(Trailing_Stop_Start_Level*P*point) && ((bid-Trailing_Stop_Distance*P*point)>(GVGet(TICKET+"VirtStop")+Trailing_Stop_Step*P*point) || GVGet(TICKET+"VirtStop")==0))
        {
         GVSet(TICKET+"VirtStop",bid-Trailing_Stop_Distance*P*point);

         if(Show_Stop_Levels_on_Chart)
            CreateText(TICKET+"VirtStop",0,TimeCurrent(),GVGet(TICKET+"VirtStop"),"VirtStop Order "+TICKET+": "+DoubleToStr(GVGet(TICKET+"VirtStop"),digits),5,"Tahoma",Blue);
        }
      if(OrderType()==OP_SELL && (OrderOpenPrice()-ask)>=(Trailing_Stop_Start_Level*P*point) && ((ask+Trailing_Stop_Distance*P*point)<(GVGet(TICKET+"VirtStop")-Trailing_Stop_Step*P*point) || GVGet(TICKET+"VirtStop")==0))
        {
         GVSet(TICKET+"VirtStop",ask+Trailing_Stop_Distance*P*point);

         if(Show_Stop_Levels_on_Chart)
            CreateText(TICKET+"VirtStop",0,TimeCurrent(),GVGet(TICKET+"VirtStop"),"VirtStop Order "+TICKET+": "+DoubleToStr(GVGet(TICKET+"VirtStop"),digits),5,"Tahoma",Blue);
        }
     }
  }
//+------------------------------------------------------------------+
//| End of TRAILING STOP for ALL POSITIONS//CURRENT SYMBOL// EA MAGIC|
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| TRAILING STOP for ALL POSITIONS // CURRENT SYMBOL // EA MAGIC    |
//+------------------------------------------------------------------+
void TrailingStopAll(string symb="")
  {
   if(Enable_Trailing_Stop && !Invisible_Stop_Levels)
     {
      string symbol=symb;
      if(symbol=="")symbol=Symbol();
      int orders=OrdersTotal()-1;
      for(int i=orders; i>=0; i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true && OrderSymbol()==symbol && OrderMagicNumber()==Magic)
           {
            int ticket=OrderTicket();
            TrailingStopOne(ticket);
           }
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void TrailingStopOne(int Ticket)
  {
   if(!Enable_Trailing_Stop || Invisible_Stop_Levels)return;
   if(OrderSelect(Ticket,SELECT_BY_TICKET) && OrderCloseTime()==0)
     {
      bool Modify=false;

      RefreshRates();
      string symbol=OrderSymbol();
      double point=MarketInfo(symbol,MODE_POINT);
      int digits=MarketInfo(symbol,MODE_DIGITS);
      double ask=MarketInfo(symbol,MODE_ASK);
      double bid=MarketInfo(symbol,MODE_BID);

      if(OrderType()==OP_BUY && (bid-OrderOpenPrice())>=(Trailing_Stop_Start_Level*P*point) && ((bid-Trailing_Stop_Distance*P*point)>(OrderStopLoss()+Trailing_Stop_Step*P*point) || OrderStopLoss()==0))
        {
         Informer("EA Journaling: Trying to modify order "+Ticket+" ...");
         if(HandleTradingEnvironment())
            Modify=OrderModify(Ticket,OrderOpenPrice(),NormalizeDouble(bid-Trailing_Stop_Distance*P*point,digits),OrderTakeProfit(),0,CLR_NONE);
         if(!Modify)Informer("EA Journaling: Unexpected Error has happened. Error Description: "+GetErrorDescription(GetLastError()));
         if(Modify)Informer("EA Journaling: Order successfully modified.");
        }
      if(OrderType()==OP_SELL && (OrderOpenPrice()-ask)>=(Trailing_Stop_Start_Level*P*point) && ((ask+Trailing_Stop_Distance*P*point)<(OrderStopLoss()-Trailing_Stop_Step*P*point) || OrderStopLoss()==0))
        {
         Informer("EA Journaling: Trying to modify order "+Ticket+" ...");
         if(HandleTradingEnvironment())
            Modify=OrderModify(Ticket,OrderOpenPrice(),NormalizeDouble(ask+Trailing_Stop_Distance*P*point,digits),OrderTakeProfit(),0,CLR_NONE);
         if(!Modify)Informer("EA Journaling: Unexpected Error has happened. Error Description: "+GetErrorDescription(GetLastError()));
         if(Modify)Informer("EA Journaling: Order successfully modified.");
        }
     }
  }
//+------------------------------------------------------------------+
//| End of TRAILING STOP for ALL POSITIONS//CURRENT SYMBOL// EA MAGIC|
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| BREAK EVEN for ALL POSITIONS // CURRENT SYMBOL // EA MAGIC       |
//+------------------------------------------------------------------+
void BreakEvenAll(string symb="")
  {
   if(Enable_Break_Even && !Invisible_Stop_Levels)
     {
      string symbol=symb;
      if(symbol=="")symbol=Symbol();
      int orders=OrdersTotal()-1;
      for(int i=orders; i>=0; i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true && OrderSymbol()==symbol && OrderMagicNumber()==Magic)
           {
            int ticket=OrderTicket();
            BreakEvenOne(ticket);
           }
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void BreakEvenOne(int Ticket)
  {
   if(!Enable_Break_Even || Invisible_Stop_Levels)return;
   if(OrderSelect(Ticket,SELECT_BY_TICKET) && OrderCloseTime()==0)
     {
      bool Modify=false;
      RefreshRates();

      string symbol=OrderSymbol();
      double point=MarketInfo(symbol,MODE_POINT);
      int digits=MarketInfo(symbol,MODE_DIGITS);
      double ask=MarketInfo(symbol,MODE_ASK);
      double bid=MarketInfo(symbol,MODE_BID);

      if(OrderType()==OP_BUY && (bid-OrderOpenPrice())>=(Break_Even_Level*P*point) && (OrderStopLoss()<NormalizeDouble(OrderOpenPrice()+Break_Even_Profit*P*point,digits) || OrderStopLoss()==0))
        {
         Informer("EA Journaling: Trying to modify order "+Ticket+" ...");
         if(HandleTradingEnvironment())
            Modify=OrderModify(Ticket,OrderOpenPrice(),NormalizeDouble(OrderOpenPrice()+Break_Even_Profit*P*point,digits),OrderTakeProfit(),0,CLR_NONE);
         if(!Modify)Informer("EA Journaling: Unexpected Error has happened. Error Description: "+GetErrorDescription(GetLastError()));
         if(Modify)Informer("EA Journaling: Order successfully modified.");
        }
      if(OrderType()==OP_SELL && (OrderOpenPrice()-ask)>=(Break_Even_Level*P*point) && (OrderStopLoss()>NormalizeDouble(OrderOpenPrice()-Break_Even_Profit*P*point,digits) || OrderStopLoss()==0))
        {
         Informer("EA Journaling: Trying to modify order "+Ticket+" ...");
         if(HandleTradingEnvironment())
            Modify=OrderModify(Ticket,OrderOpenPrice(),NormalizeDouble(OrderOpenPrice()-Break_Even_Profit*P*Point,Digits),OrderTakeProfit(),0,CLR_NONE);
         if(!Modify)Informer("EA Journaling: Unexpected Error has happened. Error Description: "+GetErrorDescription(GetLastError()));
         if(Modify)Informer("EA Journaling: Order successfully modified.");
        }
     }
  }
//+------------------------------------------------------------------+
//| End of BREAK EVEN for ALL POSITIONS // CURRENT SYMBOL // EA MAGIC|
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| HANDLE TRADING ENVIRONMENT                                       |
//+------------------------------------------------------------------+
bool HandleTradingEnvironment()
  {
   if(IsTradeAllowed())return(true);
   if(!IsConnected())
     {
      Informer("EA Journaling: Terminal is not connected to server...");
      return(false);
     }
   if(!IsTradeAllowed() && !IsTradeContextBusy())
      Informer("EA Journaling: Trade is not alowed for some reason...");
   if(IsConnected() && !IsTradeAllowed())
     {
      while(IsTradeContextBusy())
        {
         Informer("EA Journaling: Trading context is busy... Will wait a bit...");
         Sleep(Retry_Interval);
        }
     }
   if(IsTradeAllowed())
     {
      RefreshRates();
      return(true);
     }
   else
      return(false);
  }
//+------------------------------------------------------------------+
//| End of HANDLE TRADING ENVIRONMENT                                |
//+------------------------------------------------------------------+  
//+------------------------------------------------------------------+
//| ERROR DESCRIPTION                                                |
//+------------------------------------------------------------------+
string GetErrorDescription(int error)
  {
   string ErrorDescription="";
//---
   switch(error)
     {
      case 0:     ErrorDescription = "NO Error. Everything should be good :)";                                  break;
      case 1:     ErrorDescription = "No error returned, but the result is unknown";                            break;
      case 2:     ErrorDescription = "Common error";                                                            break;
      case 3:     ErrorDescription = "Invalid trade parameters";                                                break;
      case 4:     ErrorDescription = "Trade server is busy";                                                    break;
      case 5:     ErrorDescription = "Old version of the client terminal";                                      break;
      case 6:     ErrorDescription = "No connection with trade server";                                         break;
      case 7:     ErrorDescription = "Not enough rights";                                                       break;
      case 8:     ErrorDescription = "Too frequent requests";                                                   break;
      case 9:     ErrorDescription = "Malfunctional trade operation";                                           break;
      case 64:    ErrorDescription = "Account disabled";                                                        break;
      case 65:    ErrorDescription = "Invalid account";                                                         break;
      case 128:   ErrorDescription = "Trade timeout";                                                           break;
      case 129:   ErrorDescription = "Invalid price";                                                           break;
      case 130:   ErrorDescription = "Invalid stops";                                                           break;
      case 131:   ErrorDescription = "Invalid trade volume";                                                    break;
      case 132:   ErrorDescription = "Market is closed";                                                        break;
      case 133:   ErrorDescription = "Trade is disabled";                                                       break;
      case 134:   ErrorDescription = "Not enough money";                                                        break;
      case 135:   ErrorDescription = "Price changed";                                                           break;
      case 136:   ErrorDescription = "Off quotes";                                                              break;
      case 137:   ErrorDescription = "Broker is busy";                                                          break;
      case 138:   ErrorDescription = "Requote";                                                                 break;
      case 139:   ErrorDescription = "Order is locked";                                                         break;
      case 140:   ErrorDescription = "Long positions only allowed";                                             break;
      case 141:   ErrorDescription = "Too many requests";                                                       break;
      case 145:   ErrorDescription = "Modification denied because order too close to market";                   break;
      case 146:   ErrorDescription = "Trade context is busy";                                                   break;
      case 147:   ErrorDescription = "Expirations are denied by broker";                                        break;
      case 148:   ErrorDescription = "Too many open and pending orders (more than allowed)";                    break;
      case 4000:  ErrorDescription = "No error";                                                                break;
      case 4001:  ErrorDescription = "Wrong function pointer";                                                  break;
      case 4002:  ErrorDescription = "Array index is out of range";                                             break;
      case 4003:  ErrorDescription = "No memory for function call stack";                                       break;
      case 4004:  ErrorDescription = "Recursive stack overflow";                                                break;
      case 4005:  ErrorDescription = "Not enough stack for parameter";                                          break;
      case 4006:  ErrorDescription = "No memory for parameter string";                                          break;
      case 4007:  ErrorDescription = "No memory for temp string";                                               break;
      case 4008:  ErrorDescription = "Not initialized string";                                                  break;
      case 4009:  ErrorDescription = "Not initialized string in array";                                         break;
      case 4010:  ErrorDescription = "No memory for array string";                                              break;
      case 4011:  ErrorDescription = "Too long string";                                                         break;
      case 4012:  ErrorDescription = "Remainder from zero divide";                                              break;
      case 4013:  ErrorDescription = "Zero divide";                                                             break;
      case 4014:  ErrorDescription = "Unknown command";                                                         break;
      case 4015:  ErrorDescription = "Wrong jump (never generated error)";                                      break;
      case 4016:  ErrorDescription = "Not initialized array";                                                   break;
      case 4017:  ErrorDescription = "DLL calls are not allowed";                                               break;
      case 4018:  ErrorDescription = "Cannot load library";                                                     break;
      case 4019:  ErrorDescription = "Cannot call function";                                                    break;
      case 4020:  ErrorDescription = "Expert function calls are not allowed";                                   break;
      case 4021:  ErrorDescription = "Not enough memory for temp string returned from function";                break;
      case 4022:  ErrorDescription = "System is busy (never generated error)";                                  break;
      case 4050:  ErrorDescription = "Invalid function parameters count";                                       break;
      case 4051:  ErrorDescription = "Invalid function parameter value";                                        break;
      case 4052:  ErrorDescription = "String function internal error";                                          break;
      case 4053:  ErrorDescription = "Some array error";                                                        break;
      case 4054:  ErrorDescription = "Incorrect series array using";                                            break;
      case 4055:  ErrorDescription = "Custom indicator error";                                                  break;
      case 4056:  ErrorDescription = "Arrays are incompatible";                                                 break;
      case 4057:  ErrorDescription = "Global variables processing error";                                       break;
      case 4058:  ErrorDescription = "Global variable not found";                                               break;
      case 4059:  ErrorDescription = "Function is not allowed in testing mode";                                 break;
      case 4060:  ErrorDescription = "Function is not confirmed";                                               break;
      case 4061:  ErrorDescription = "Send mail error";                                                         break;
      case 4062:  ErrorDescription = "String parameter expected";                                               break;
      case 4063:  ErrorDescription = "Integer parameter expected";                                              break;
      case 4064:  ErrorDescription = "Double parameter expected";                                               break;
      case 4065:  ErrorDescription = "Array as parameter expected";                                             break;
      case 4066:  ErrorDescription = "Requested history data in updating state";                                break;
      case 4067:  ErrorDescription = "Some error in trading function";                                          break;
      case 4099:  ErrorDescription = "End of file";                                                             break;
      case 4100:  ErrorDescription = "Some file error";                                                         break;
      case 4101:  ErrorDescription = "Wrong file name";                                                         break;
      case 4102:  ErrorDescription = "Too many opened files";                                                   break;
      case 4103:  ErrorDescription = "Cannot open file";                                                        break;
      case 4104:  ErrorDescription = "Incompatible access to a file";                                           break;
      case 4105:  ErrorDescription = "No order selected";                                                       break;
      case 4106:  ErrorDescription = "Unknown symbol";                                                          break;
      case 4107:  ErrorDescription = "Invalid price";                                                           break;
      case 4108:  ErrorDescription = "Invalid ticket";                                                          break;
      case 4109:  ErrorDescription = "EA is not allowed to trade is not allowed. ";                             break;
      case 4110:  ErrorDescription = "Longs are not allowed. Check the expert properties";                      break;
      case 4111:  ErrorDescription = "Shorts are not allowed. Check the expert properties";                     break;
      case 4200:  ErrorDescription = "Object exists already";                                                   break;
      case 4201:  ErrorDescription = "Unknown object property";                                                 break;
      case 4202:  ErrorDescription = "Object does not exist";                                                   break;
      case 4203:  ErrorDescription = "Unknown object type";                                                     break;
      case 4204:  ErrorDescription = "No object name";                                                          break;
      case 4205:  ErrorDescription = "Object coordinates error";                                                break;
      case 4206:  ErrorDescription = "No specified subwindow";                                                  break;
      case 4207:  ErrorDescription = "Some error in object function";                                           break;
      default:    ErrorDescription = "No error or error is unknown";
     }
   return(ErrorDescription);
  }
//+------------------------------------------------------------------+
//| End of ERROR DESCRIPTION                                         |
//+------------------------------------------------------------------+

